--/*************************************************************************
--   (c) Copyright IBM Corp. 2007 All rights reserved.
--   
--   The following sample of source code ("Sample") is owned by International 
--   Business Machines Corporation or one of its subsidiaries ("IBM") and is 
--   copyrighted and licensed, not sold. You may use, copy, modify, and 
--   distribute the Sample in any form without payment to IBM, for the purpose of 
--   assisting you in the development of your applications.
--   
--   The Sample code is provided to you on an "AS IS" basis, without warranty of 
--   any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
--   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
--   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
--   not allow for the exclusion or limitation of implied warranties, so the above 
--   limitations or exclusions may not apply to you. IBM shall not be liable for 
--   any damages you suffer as a result of using, copying, modifying or 
--   distributing the Sample, even if IBM has been advised of the possibility of 
--   such damages.
-- *************************************************************************
--                                                                          
-- SAMPLE FILE NAME: xmldecomposition.db2                                          
--
-- PURPOSE: To demonstrate annotated XML schema decomposition and 
--          specifying the order of insertion of rows into target tables.
--
-- USER SCENARIO:
--	   A bookstore has some XML documents which contain descriptive 
--         information about a book that is for sale. The store owner needs to 
--         store these details in different relational tables with 
--         referential constraints for easy retreival of data.
--         Importing of the XML data in an order can be done using annotated 
--         XML schema decomposition. 
--
-- SOLUTION:
--         The user must have an annotated schema based on which the instance document 
--         can be decomposed. Once a valid annotated schema for the instance document  
--         is ready, it needs to be registered with the XML schema repository with 
--         the decomposition option enabled. Also, the tables in which the data will be 
--         decomposed must exist before the schema is registered. The user can 
--         decompose the instance document and store the data in the relational 
--         tables using annotated XML Decomposition.
--
--         The following relational tables will be used to store the decomposed XML data:
--		a) ADMIN.BOOK_AUTHOR
--		b) XDB.BOOK_CONTENTS
--              c) XDB.BOOK_AVAIL
--
--         The XML schema bookdetail.xsd specifies that the data in the instance document
--         bookdetail.xml will be inserted into the tables listed above. The annotations 
--         in this XML schema specify that the SQL schema for the table BOOK_AUTHOR will
--         be ADMIN, and for the other tables it will be defaultSQLSchema which is XDB.
--         This annotation can be used only once in the xmlschema.
--
--         The XML data is inserted into tables in the order specified using the
--         "rowSetOperationOrder" annotation in the schema. 
--
--         The text within the element "authorId" will be of type INTEGER and will be inserted
--         into the column "AUTHID" of the TABLE "BOOK_AUTHOR". A condition annotation has
--         been put here to check if the value of "AUTHID" is a number between 1 and 999. If the 
--         value passes the CONDITION check, then it is inserted into the table.
--         The ISBN column of BOOK_AUTHOR  table is the primary key and it is referenced by
--         ISBN column of the BOOK_CONTENTS and BOOK_AVAIL tables. The text in the "isbn" 
--         attribute of the "chapter" element will be inserted into the column "ISBN" 
--         of the tables "BOOK_AUTHOR", "BOOK_CONTENTS", and "BOOK_AVAIL" in order as 
--         this is the primary key in one table and foreign key in other tables. 
--         The text in the "title" element will be inserted into the BOOK_TITLE column of the
--         "BOOK_AUTHOR" and "BOOK_AVAIL" tables. The text in the "chapter" element will be 
--         inserted into different tables and different columns. The contents of the element 
--         "chaptercontents" will be inserted into the column "CHPTCONTENT" of table
--         "BOOK_CONTENTS". The db2-xdb:truncate decomposition annotation indicates
--         that the value to be inserted into the column will be trucated if its size is larger 
--         than the column size. Another annotation that is used for this element is
--         db2-xdb:contentHandling, which indicates that the concatenation of this element's 
--         character data (including character content of CDATA sections) with the character data 
--         in this element's descendants, in document order, will be inserted into the table.
--        
--  PREREQUISITE:
--        The instance document and the annotated schema must exist in the same 
--        directory as the sample. Copy bookdetail.xml, bookdetail.xsd, 
--        setupfordecomposition.db2 and cleanupfordecomposition.db2 from directory 
--        <install_path>/sqllib/samples/xml/data in UNIX and
--        <install_path>\sqllib\samples\xml\data in Windows to the working directory. 
--                                                                          
--  EXECUTION:    i)   db2 -tvf setupfordecomposition.db2 (setup script 
--                     to create the required tables)
--                ii)  db2 -tvf xmldecomposition.db2 (execute the sample)
--                iii) db2 -tvf cleanupfordecomposition.db2 (clean up 
--                     script to drop all the objects created)
--                                                                          
--  INPUTS:       NONE
--                                                                          
--  OUTPUTS:      Decomposition of XML document according to the dependencies 
--                specified in the annotated XML schema.
--                                                                          
--  OUTPUT FILE:  xmldecomposition.out (available in the online documentation)      
--                                     
--  SQL STATEMENTS USED:                                                    
--        REGISTER XMLSCHEMA                                                  
--        COMPLETE XMLSCHEMA      
--        DECOMPOSE XML DOCUMENT                                          
--        CREATE   
--        SELECT 
--        DROP                                          
--  
-- *************************************************************************
-- For more information about the command line processor (CLP) scripts,     
-- see the README file.                                                     
-- For information on using SQL statements, see the SQL Reference.          
--                                                                          
-- For the latest information on programming, building, and running DB2     
-- applications, visit the DB2 application development website:             
-- http://www.software.ibm.com/data/db2/udb/ad                              
-- *************************************************************************
--
--  SAMPLE DESCRIPTION                                                      
--
-- *************************************************************************
-- 1. Register the annotated XML schema.
-- 2. Decompose the XML document using the registered XML schema.
-- 3. Select data from the relational tables to see decomposed data.
-- *************************************************************************

-- /*************************************************************************
--    SETUP -- execute setupfordecomposition.db2       
-- **************************************************************************/

-- setupfordecomposition.db2 contains DDL statements to create tables with
-- referential constraints.

-- Connect to the SAMPLE database 
CONNECT TO SAMPLE;

-- /*************************************************************************
-- 1. Register the annotated XML schema.
-- *************************************************************************/

-- Register the XML schema document containing annotations to store the data 
-- into relational tables as per the order specified. 
REGISTER XMLSCHEMA 'http://book.com/bookdetail.xsd' FROM 'bookdetail.xsd' AS xdb.bookdetail;

-- Complete XML schema registration
COMPLETE XMLSCHEMA xdb.bookdetail ENABLE DECOMPOSITION;

-- Check catalog tables for information regarding the newly registered schema.
SELECT status, decomposition, decomposition_version 
                       FROM SYSIBM.SYSXSROBJECTS 
                       where XSROBJECTNAME = 'BOOKDETAIL';

-- /*************************************************************************
-- 2. Decompose the XML document using the registered XML schema.
-- *************************************************************************/

-- Decompose the XML document
DECOMPOSE XML DOCUMENT bookdetail.xml
		       XMLSCHEMA xdb.bookdetail
	               VALIDATE;	

-- /*************************************************************************
-- 3. Select data from the relational tables to see decomposed data.
-- *************************************************************************/

-- check the results of the decomposition
SELECT * FROM ADMIN.BOOK_AUTHOR;
SELECT * FROM XDB.BOOK_AVAIL;
SELECT * FROM XDB.BOOK_CONTENTS;

-- Perform Commit
COMMIT;

-- /*************************************************************************
--    CLEANUP -- execute cleanupfordecomposition.db2
-- **************************************************************************/

-- Reset the connection
CONNECT RESET;

