//***************************************************************************
//   (c) Copyright IBM Corp. 2007 All rights reserved.
//   
//   The following sample of source code ("Sample") is owned by International 
//   Business Machines Corporation or one of its subsidiaries ("IBM") and is 
//   copyrighted and licensed, not sold. You may use, copy, modify, and 
//   distribute the Sample in any form without payment to IBM, for the purpose of 
//   assisting you in the development of your applications.
//   
//   The Sample code is provided to you on an "AS IS" basis, without warranty of 
//   any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
//   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
//   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
//   not allow for the exclusion or limitation of implied warranties, so the above 
//   limitations or exclusions may not apply to you. IBM shall not be liable for 
//   any damages you suffer as a result of using, copying, modifying or 
//   distributing the Sample, even if IBM has been advised of the possibility of 
//   such damages.
// **************************************************************************
//                                                                          
// SAMPLE FILE NAME: XmlDecomposition.java                                          
//
// PURPOSE: To demonstrate annotated XML schema decomposition and 
//          specifying the order of insertion of rows into target tables.
//
// USER SCENARIO:
//	   A bookstore has some XML documents which contain descriptive 
//         information about a book that is for sale. The store owner needs to 
//         store these details in different relational tables with referential 
//         constraints for easy retreival of data.
//         Importing of the XML data in an order can be done using annotated 
//         XML schema decomposition. 
//
// SOLUTION:
//         The user must have an annotated schema based on which the instance document 
//         can be decomposed. Once a valid annotated schema for the instance document  
//         is ready, it needs to be registered with the XML schema repository with 
//         the decomposition option enabled. Also, the tables in which the data will be 
//         decomposed must exist before the schema is registered. The user can 
//         decompose the instance document and store the data in the relational 
//         tables using annotated XML Decomposition.
//
//         The following relational tables will be used to store the decomposed XML data:
//		a) ADMIN.BOOK_AUTHOR
//		b) XDB.BOOK_CONTENTS
//              c) XDB.BOOK_AVAIL
//
//         The XML schema bookdetail.xsd specifies that the data in the instance document
//         bookdetail.xml will be inserted into the tables listed above. The annotations 
//         in this XML schema specify that the SQL schema for the table BOOK_AUTHOR will
//         be ADMIN, and for the other tables it will be defaultSQLSchema which is XDB.
//         This annotation can be used only once in the xmlschema.
//
//         The XML data is inserted into tables in the order specified using the
//         "rowSetOperationOrder" annotation in the schema. 
//
//         The text within the element "authorId" will be of type INTEGER and will be inserted
//         into the column "AUTHID" of the TABLE "BOOK_AUTHOR". A condition annotation has
//         been put here to check if the value of "AUTHID" is a number between 1 and 999. If the 
//         value passes the CONDITION check, then it is inserted into the table.
//         The ISBN column of BOOK_AUTHOR  table is the primary key and it is referenced by
//         ISBN column of the BOOK_CONTENTS and BOOK_AVAIL tables. The text in the "isbn" 
//         attribute of the "chapter" element will be inserted into the column "ISBN" 
//         of the tables "BOOK_AUTHOR", "BOOK_CONTENTS", and "BOOK_AVAIL" in order as 
//         this is the primary key in one table and foreign key in other tables. 
//         The text in the "title" element will be inserted into the BOOK_TITLE column of the
//         "BOOK_AUTHOR" and "BOOK_AVAIL" tables. The text in the "chapter" element will be 
//         inserted into different tables and different columns. The contents of the element 
//         "chaptercontents" will be inserted into the column "CHPTCONTENT" of table
//         "BOOK_CONTENTS". The db2-xdb:truncate decomposition annotation indicates
//         that the value to be inserted into the column will be trucated if its size is larger 
//         than the column size. Another annotation that is used for this element is
//         db2-xdb:contentHandling, which indicates that the concatenation of this element's 
//         character data (including character content of CDATA sections) with the character data 
//         in this element's descendants, in document order, will be inserted into the table.
//        
//  PREREQUISITE:
//        The instance document and the annotated schema must exist in the same 
//        directory as the sample. Copy bookdetail.xml, bookdetail.xsd, 
//        setupfordecomposition.db2 and cleanupfordecomposition.db2 from directory 
//        <install_path>/sqllib/samples/xml/data in UNIX and
//        <install_path>\sqllib\samples\xml\data in Windows to the working directory. 
//                                                                          
//  EXECUTION:    i)   db2 -tvf setupfordecomposition.db2 (setup script
//                     to create the required tables.
//                ii)  javac xmldecomposition.java (Compile the sample)
//                iii) java XmlDecomposition (Run the sample)
//                iv)  db2 -tvf cleanupfordecomposition.db2 (clean up
//                     script to drop all the objects created)
//
//  INPUTS:       NONE
//
//  OUTPUTS:      Decomposition of XML document according to the dependencies
//                specified in the annotated XML schema.
//
//  OUTPUT FILE:  XmlDecomposition.out (available in the online documentation)
//
// SQL STATEMENTS USED:
//         REGISTER XMLSCHEMA
//         COMPLETE XMLSCHEMA
//         SELECT
//         DECOMPOSE
//
// Classes used from Util.java are:
//         Db
//         Data
//         JdbcException
//         Statement
//         ResultSet
//
//***************************************************************************
//
// For more information on the sample programs, see the README file.
//
// For information on developing JDBC applications, see the Application
// Development Guide.
//
// For information on using SQL statements, see the SQL Reference.
//
// For the latest information on programming, compiling, and running DB2
// applications, visit the DB2 application development website at
//     http://www.software.ibm.com/data/db2/udb/ad
//***************************************************************************
//
//  SAMPLE DESCRIPTION
//
// *****************************************************************************
// 1. Register the annotated XML schema.
// 2. Decompose the XML document using the registered XML schema.
// 3. Select data from the relational tables to see decomposed data.
// *****************************************************************************

import java.lang.*;
import java.sql.*;
import java.io.*;

class XmlDecomposition
{
  public static void main(String argv[])
  {
    try
    {
      Db db = new Db(argv);

      System.out.println();
      System.out.println("THIS SAMPLE SHOWS HOW TO DECOMPOSE DATA FROM AN XML DOCUMENT INTO RELATIONAL TABLES");

      // connect to the 'sample' database
      db.connect();

      XmlDecompose(db.con);

      // disconnect from the 'sample' database
      db.disconnect();
    }
    catch (Exception e)
    {
      JdbcException jdbcExc = new JdbcException(e);
      jdbcExc.handle();
    }
  } // main
  
  static void XmlDecompose(Connection con)
  {
    try
      {
         String dbname;
         String RelSchema;
         String SchemaName;
         String SchemaLocation;
         String PrimaryDocument;
         String xmlfilename;
         int shred = 1;
         String Status;
         String Decomposition;
         String Decomposition_version;
         boolean  xmlRegister = false;
         boolean  xmlAdd = false;
         boolean  xmlComplete = false;
         boolean  xmlDecomp = false;
    
         RelSchema = "xdb";
         SchemaName = "bookdetail";
         SchemaLocation = "http://bookdetail/schema1";
         PrimaryDocument = "bookdetail.xsd";
         xmlfilename = "bookdetail.xml";
 
         // Register the XML Schema to the XSR.
         CallableStatement callStmt = con.prepareCall("CALL SYSPROC.XSR_REGISTER(?,?,?,?,NULL)");
         File xsdfile = new File(PrimaryDocument);
         FileInputStream xsdfileis = new FileInputStream(xsdfile);

         callStmt.setString(1, RelSchema );
         callStmt.setString(2, SchemaName );
         callStmt.setString(3, SchemaLocation );
         callStmt.setBinaryStream(4, xsdfileis, (int)xsdfile.length() );
         callStmt.execute();
         xsdfileis.close();
         callStmt.close();
         System.out.println("**** CALL SYSPROC.XSR_REGISTER SUCCESSFULLY");

         // Complete the Schema registration with Validate Option true.
         callStmt = con.prepareCall("CALL SYSPROC.XSR_COMPLETE(?,?,NULL,?)");
         callStmt.setString(1, RelSchema );
         callStmt.setString(2, SchemaName );
         callStmt.setInt(3, shred );
         callStmt.execute();
         callStmt.close();
         System.out.println("**** CALL SYSPROC.XSR_COMPLETE SUCCESSFULLY");

         // Check the status of the XSR object registered.
         Statement stmt = con.createStatement();
         ResultSet rs = stmt.executeQuery(
           "SELECT status, decomposition, decomposition_version FROM SYSIBM.SYSXSROBJECTS WHERE XSROBJECTNAME = 'BOOKDETAIL'");
   
         while(rs.next())
          {
            Status = rs.getString(1);
            Decomposition = rs.getString(2);
            Decomposition_version = rs.getString(3);

            System.out.println("\nStatus          : " +
                               Data.format(Status, 5) + "\n" +
                               "Decomposition      : " +
                               Data.format(Decomposition, 5) + "\n" +
                               "Version : " +
                               Data.format(Decomposition_version, 5));
          } 
        rs.close();       

        // Decompose the XML document by calling the SYSPROC.XDBDECOMPXML
        callStmt = con.prepareCall("CALL SYSPROC.XDBDECOMPXML(?,?,?,?,?, NULL, NULL, NULL)");
        File xmlfile = new File(xmlfilename);
        FileInputStream xmlfileis = new FileInputStream(xmlfile);
        callStmt.setString(1, RelSchema );
        callStmt.setString(2, SchemaName );
        callStmt.setBinaryStream(3, xmlfileis, (int)xmlfile.length() );
        callStmt.setString(4, SchemaName );
        callStmt.setInt(5, shred);
        callStmt.execute();
        xmlfileis.close();
        callStmt.close();
        System.out.println("**** CALL SYSPROC.XDBDECOMPXML SUCCESSFULLY");

        // Read Data from the tables, where the data is stored after decomposition.
        SelectFromTables(con);
    
      }
    catch (Exception e)
      {
        JdbcException jdbcExc = new JdbcException(e, con);
        jdbcExc.handle();
      }
  }
  static void SelectFromTables(Connection con)
  {
    try
      {
        String isbn = "";
        int chptnum = 0;
        String chpttittle = "";
        String chptcontent = "";
        int authid = 0;
        String book_title = "";
        String status = "";
        String decompose = "";
        String decomp_version = "";

        Statement stmt = con.createStatement();
        ResultSet rs = stmt.executeQuery(
          "SELECT isbn, chptnum, chpttittle, chptcontent FROM XDB.BOOK_CONTENTS");

        while (rs.next())
          {
            isbn = rs.getString(1);
            chptnum = rs.getInt(2);
            chpttittle = rs.getString(3);
            chptcontent = rs.getString(4);
 
            System.out.println("\nISBN          : " +
                               Data.format(isbn, 13) + "\n" +
                               "CHAPTER NUMBER  : " +
                               Data.format(chptnum, 5) + "\n" +
                               "Chapter Title   : " +
                               Data.format(chpttittle, 50) + "\n" +
                               "Chapter Content : " +
                               Data.format(chptcontent, 1000));
          } 
  
        // Select data from the ADMIN.BOOK_AUTHOR TABLE.
        rs = stmt.executeQuery("SELECT authid, isbn, book_title FROM ADMIN.BOOK_AUTHOR");
       
        while(rs.next())
          {
            authid = rs.getInt(1);
            isbn = rs.getString(2);
            book_title = rs.getString(3);
            
            System.out.println("\nAuthor ID   : " +
                              Data.format(authid, 5) + "\n" +
                              "ISBN       : " + 
                              Data.format(isbn, 13) + "\n" +
                              "Book Title : " +
                              Data.format(book_title, 50));
          } 
   
        // Select data from the XDB.BOOK_AVAIL TABLE.
        rs = stmt.executeQuery("SELECT isbn, book_title FROM XDB.BOOK_AVAIL");

        while(rs.next())
          {
            
            isbn = rs.getString(1);
            book_title = rs.getString(2);

            System.out.println("\nISBN       : " +
                              Data.format(isbn, 13) + "\n" +
                              "Book Title : " +
                              Data.format(book_title, 50));
          }
        rs.close();
 
      }
    catch (Exception e)
      {
        JdbcException jdbcExc = new JdbcException(e, con);
        jdbcExc.handle();
      } //Try Block
   }  //SelectFromTable
}  //XmlDecomposition Class  
   
